#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 17-7-8
# @Author  : JCai
# @desc    :

import os
import django
from django.db import transaction

from info_sync.views.listing_sync_info import info_variation_change_to_sql, info_base_change_to_sql, \
    info_more_change_to_sql

os.environ['DJANGO_SETTINGS_MODULE'] = 'EBayClub.settings'
django.setup()

from datetime import datetime, timedelta

import pytz
from django.utils import timezone

from info_sync.interface.trading_inter import MyEbay
from info_sync.models.listing_info_models import EBayListing, EBayListingVariations
from utils.logger import logStatusBegin, logStatusEnd, logInfo
from info_sync.models.account_models import EBayAccount
from news_push.tasks import listing_info_replace
from utils.util import str_to_date



def need_update_info(item_info):
    item_id = item_info.get('ItemID', '')
    watch_count = int(item_info.get('WatchCount', 0))
    quantity = int(item_info.get('Quantity', 0))
    listing_status = item_info.get('SellingStatus', {}).get('ListingStatus', '')
    quantity_sold = int(item_info.get('SellingStatus', {}).get('QuantitySold', 0))
    current_price = float(item_info.get('SellingStatus', {}).get('CurrentPrice', {}).get('value', '0.0'))
    end_time = str_to_date(item_info.get('ListingDetails', {}).get('EndTime', ''))
    variations = item_info.get('Variations', {}).get('Variation', [])
    if type(variations) == type({}):  # if only have a variations  ,the resutl is a dict
        variations = [variations]

    variation_list = []
    variation_count = 0
    for variation in variations:
        specifics = variation.get('VariationSpecifics', {}).get('NameValueList', [])
        if type(specifics) == type({}):
            specifics = [specifics]
        variation_sku = variation.get('SKU', '')
        variation_price = float(variation.get('StartPrice', {}).get('value', '0.0'))
        variation_quantity = int(variation.get('Quantity', '0'))
        variation_quantity_sold = int(variation.get('SellingStatus', {}).get('QuantitySold', '0'))
        variation_info = {}
        variation_listings = EBayListingVariations.objects.filter(item=item_id, seller_sku=variation_sku)
        if variation_listings:
            variation_listing = variation_listings.first()
            if variation_listing:
                if variation_price != variation_listing.current_rice:
                    variation_info['current_rice'] = variation_price
                if variation_quantity != variation_listing.quantity:
                    variation_info['quantity'] = variation_quantity
                if variation_quantity_sold != variation_listing.quantity_sold:
                    variation_info['quantity_sold'] = variation_quantity_sold
                if specifics != variation_listing.variation_specifics:
                    variation_info['variation_specifics'] = specifics
                variation_listings.update(**variation_info)
                if len(variation_info):
                    variation_info['operate'] = 'update'
        # else:
        #     variation_info['current_rice'] = variation_price
        #     variation_info['quantity'] = variation_quantity
        #     variation_info['quantity_sold'] = variation_quantity_sold
        #     variation_info['variation_specifics'] = specifics
        #     EBayListingInfoVariations.objects.create(
        #         item=item_id,
        #         seller_sku=variation_sku,
        #         **variation_info
        #     )
        #     variation_info['operate'] = 'add'
        if len(variation_info):
            variation_info['item'] = item_id
            variation_info['seller_sku'] = variation_sku
            variation_list.append(variation_info)
        variation_count += 1

    base_info = {}
    base_listings = EBayListing.objects.filter(item=item_id)
    if base_listings:
        base_listing = base_listings.first()
        if base_listing:
            if watch_count != base_listing.watch_count:
                base_info['watch_count'] = watch_count
            if quantity != base_listing.quantity:
                base_info['quantity'] = quantity
            if listing_status != base_listing.listing_status:
                base_info['listing_status'] = listing_status
            if quantity_sold != base_listing.quantity_sold:
                base_info['quantity_sold'] = quantity_sold
            if end_time != base_listing.end_time:
                base_info['end_time'] = end_time
            if variation_count != base_listing.variation:
                base_info['variation'] = variation_count
            if current_price != base_listing.current_price:
                base_info['current_price'] = current_price
            base_listings.update(**base_info)
            if len(base_info):
                base_info['operate'] = 'update'
                base_info['item_id'] = item_id
    else:
        return item_id, base_info, variation_list
    #     base_info['watch_count'] = watch_count
    #     base_info['quantity'] = quantity
    #     base_info['listing_status'] = listing_status
    #     base_info['quantity_sold'] = quantity_sold
    #     base_info['end_time'] = end_time
    #     base_info['variation'] = variation_count
    #     base_info['item'] = item_id
    #     EBayListingInfoBase.objects.create(
    #         **base_info
    #     )
    #     base_info = {}
    # if listing_status != 'Active':
    #     return '', base_info, variation_list
    return '', base_info, variation_list
    # return item_id, base_info, variation_list


def listing_info_change_sync(store_obj, from_time, to_time):
    logStatusBegin('Listing Info Change Sync Begin')
    day_now = datetime.now(pytz.UTC) + timedelta(minutes=2)
    mod_time_from = str(day_now - timedelta(hours=1)).split('.')[0].replace(' ', 'T') + 'Z'
    mod_time_to = str(day_now).split('.')[0].replace(' ', 'T') + 'Z'
    call_data = {
        'EndTimeFrom': from_time,
        'EndTimeTo': to_time,
        'ModTimeFrom': mod_time_from,
        'ModTimeTo': mod_time_to,
        'IncludeWatchCount': True,
        'IncludeVariationSpecifics': True,
        'OutputSelector': ['ItemArray.Item.ItemID',
                           'ItemArray.Item.WatchCount',
                           'ItemArray.Item.SellingStatus.ListingStatus',
                           'ItemArray.Item.SellingStatus.QuantitySold',
                           'ItemArray.Item.SellingStatus.CurrentPrice.value',
                           'ItemArray.Item.ListingDetails.EndTime',
                           'ItemArray.Item.Quantity',
                           'ItemArray.Item.ListingType',
                           'ItemArray.Item.Variations'],
        'DetailLevel': 'ReturnAll',
    }
    token = store_obj.get('Token', '')
    item_array = []
    if not token or len(token) <= 0:
        logInfo('Sync ERROR:\t%s\n\t%s' % (store_obj.get('name', ''), 'Token is Empty!'))
        logStatusEnd('Listing Info Change Sync END')
        return item_array

    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    try:
        status, message, listing_items = my_ebay.get_seller_events(call_data)
        if not status:
            logInfo('Sync Info: %s\n\t%s' % (store_obj.get('name', ''), message))
            logStatusEnd('Listing Info Change Sync END')
            return item_array

        if type(listing_items) == type({}):
            listing_items = [listing_items]

        logInfo('Sync Info: Update Number\t%s' % str(len(listing_items)))
        for each in listing_items:
            try:
                item_id, base_info, variation_list = need_update_info(each)
                if len(item_id):
                    item_array.append(item_id)
                else:
                    if len(base_info) or len(variation_list):
                        listing_info_replace.delay(base_info, variation_list)

            except Exception, e:
                logInfo('SQL ERROR:\t%s\n\t%s' % (store_obj.get('name', ''), str(e)))
    except Exception, e:
        logInfo('Sync ERROR:\t%s\n\t%s' % (store_obj.get('name', ''), str(e)))

    logStatusEnd('Listing Info Change Sync END')

    return item_array


def listing_more_selector_keys():
    return ['Item.ItemID',
            'Item.Site',
            'Item.SellingStatus.ListingStatus',
            'Item.ListingType',
            'Item.SKU',
            'Item.Title',
            'Item.SubTitle',
            'Item.Currency',
            'Item.SellingStatus.CurrentPrice',
            'Item.StartPrice',
            'Item.ReservePrice',
            'Item.BuyItNowPrice',
            'Item.ListingDuration',
            'Item.Quantity',
            'Item.SellingStatus.QuantitySold',
            'Item.ListingDetails.StartTime',
            'Item.ListingDetails.EndTime',
            'Item.Variations',
            'Item.Country',
            'Item.Location',
            'Item.PrimaryCategory',
            'Item.SecondaryCategory',
            'Item.Storefront',
            # 'Item.VariationSpecificName'
            'Item.ConditionID',
            'Item.Description',
            'Item.PictureDetails.PictureURL',
            'Item.DispatchTimeMax',
            'Item.ShippingDetails.ShippingServiceOptions',
            'Item.ShippingDetails.InternationalShippingServiceOption',
            'Item.ShippingDetails.ExcludeShipToLocation',
            'Item.ReturnPolicy',
            'Item.BuyerRequirementDetails',
            'Item.ItemSpecifics.NameValueList',
            'Item.Variations',
            'Item.PaymentMethods',
            'Item.PayPalEmailAddress',
            'Item.PaymentInstructions',
            'Item.PostalCode']


def need_update_more(store_id, item_id, item_info):
    open_date = str_to_date(item_info.get('ListingDetails', {}).get('StartTime', ''))
    variations = item_info.get('Variations', {}).get('Variation', [])
    if type(variations) == type({}):  # if only have a variations  ,the resutl is a dict
        variations = [variations]
    variation_list, variation_count = info_variation_change_to_sql(item_id, variations, open_date)

    base_info = info_base_change_to_sql(item_id, item_info, store_id, variation_count, open_date)
    more_info = info_more_change_to_sql(item_id, item_info)
    return base_info, more_info, variation_list


def listing_more_info_sync(item_ids, store_obj):
    logStatusBegin('Listing More Info Sync Begin')
    token = store_obj.get('Token', '')
    store_name = store_obj.get('name', '')

    if not token or len(token) <= 0:
        logInfo('sync error:\t%s\n\t%s' % (store_name, 'Token is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return
    store_id = store_obj.get('account_id', '')
    if not store_id or store_id <= 0:
        logInfo('sync error:\t%s\n\t%s' % (store_name, 'Store ID is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return
    call_data = {
        'IncludeVariations': True,
        'IncludeItemSpecifics': True,
        'GranularityLevel': 'Fine',
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_more_selector_keys(),
    }
    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})

    count = len(item_ids)
    number = 1
    for item_id in item_ids:
        logInfo('%5d/%5d\tItemID:%s' % (number, count, item_id))

        try:
            call_data['ItemID'] = item_id
            status, message, listing_items = my_ebay.get_item(call_data)
            number += 1
            if not status:
                logInfo('sync info: %s\n%s' % (store_name, message))
                logStatusEnd('Listing Base Info Sync END')
                continue

            try:
                base_info, more_info, variation_list = need_update_more(store_id, item_id, listing_items)
                if len(base_info) or len(more_info) or len(variation_list):
                    listing_info_replace.delay(base_info, variation_list, more_info)
            except Exception, e:
                logInfo('SQL error:\t%s\n\t%s' % (store_name, str(e)))

        except Exception, e:
            logInfo('sync error:\t%s\n\t%s' % (store_name, str(e)))
    logStatusEnd('Listing More Info Sync END')


def ebay_listing_sync_per_hour(store):
    logStatusBegin('Listing Sync Begin:%s' % store.get('name', ''))
    from_time = str(datetime.now(pytz.UTC) - timedelta(days=3)).split('.')[0].replace(' ', 'T') + 'Z'
    to_time = str(datetime.now(pytz.UTC) + timedelta(days=33)).split('.')[0].replace(' ', 'T') + 'Z'
    item_list = listing_info_change_sync(store, from_time, to_time)

    # item_list = ['282369331498']
    listing_more_info_sync(item_list, store)
    logStatusEnd('Listing Sync End:%s' % store.get('name', ''))


